package main

import (
	"cidebug/pkg/iowrappers"
	"fmt"
	"github.com/gorilla/websocket"
	"log"
	"net/http"
)

var upgrader = websocket.Upgrader{
	ReadBufferSize:  1024,
	WriteBufferSize: 1024,
}

func handleWebSocket(w http.ResponseWriter, r *http.Request,
	handler func(w http.ResponseWriter, r *http.Request, websockerConnection *iowrappers.WebSocketConn)) {
	conn, err := upgrader.Upgrade(w, r, nil)
	if err != nil {
		log.Println("Error upgrading to WebSocket:", err)
		return
	}
	wsConn := iowrappers.NewWebSocketConn(conn)
	defer wsConn.Close()

	handler(w, r, wsConn)
}

type WebSocketService struct {
	handler func(w http.ResponseWriter, r *http.Request, conn *iowrappers.WebSocketConn)
}

func (endpoint *WebSocketService) handle(w http.ResponseWriter, r *http.Request) {
	handleWebSocket(w, r, endpoint.handler)
}

var client *iowrappers.WebSocketConn
var agent *iowrappers.WebSocketConn

func main() {
	// Connect to TCP server

	clientClosed := make(chan bool)
	serverAvailable := make(chan bool)

	registrationService := WebSocketService{
		handler: func(w http.ResponseWriter, r *http.Request, conn *iowrappers.WebSocketConn) {
			log.Println("Got registration connection")
			agent = conn
			serverAvailable <- true
			<-clientClosed
		},
	}
	clientService := WebSocketService{
		handler: func(w http.ResponseWriter, r *http.Request, conn *iowrappers.WebSocketConn) {
			log.Println("Got client connection")
			client = conn
			if agent == nil {
				log.Println("There is no agent yet,, closing")
				return
			}
			iowrappers.SynchronizeStreams(client, agent)
			clientClosed <- true
		},
	}

	http.HandleFunc("/register", registrationService.handle)
	http.HandleFunc("/client", clientService.handle)

	// Start HTTP server
	fmt.Println("WebSocket server listening on :8000")
	log.Fatal(http.ListenAndServe(":8000", nil))
}
